%clear all;
% Simulation of concentration and current in a redoxcycling Sensor
%
% This script will simulate in two dimensions the transient behavior
% of the redox species when stepping the voltage from zero to E.
% The explicit differences method with zero-order current approximation
% will be used. Analyt intake and variable sets of AE will be regarded
% in the calculation.
% The calculation is reduced to one form of the mediator, which is the
% oxidized form. Thus all concentration outputs will represent the
% concentration of the oxidzed form only.
%
% This is the calculation for cartesian coordinates, assuming the
% geometry to be symetrical along the z axis. Thus the geometry can be
% reduced to two coordinates (x,z).
%
% The calculation scheme is derived from the simslit program of Mr.
% Neudeck.
%
% Conventions:
% all values are converted into SI-units before start of calculation
% x-axis length of channel
% y-axis width of channel (=width of AE)
% z-axis heigth of channel (=distance of AE)
%
% !!! remember !!! : in Matlab the indexing convention is A(x,z), that is
% if you like your colums to be x and your rows to be z.
% ===> correct is i.e. c_old(i,j)

%%%%%%%%%%%%%%%%%
%%% CONSTANTS %%%
R=8.31451;                    % molar gas constant    [R] = J / mol / K
F=96485.3;                  % faraday constant      [F] = A s / mol
T=293.15;                   % temperature           [T] = K
FRT = F/R/T;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%% OPTIONS DIALOGS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% analyte input 1..yes  0..no
button = questdlg('Do you want to simulate analyte input to the sensor?',...
                  'Analyte input...','Yes','No','Yes');
if strcmp(button,'Yes')
    analyte_yes_no = 1;
elseif strcmp(button,'No')
    analyte_yes_no = 0;
end;

% recording of concentration profile over time in an avi movie
button = questdlg('Do you want to record the results as avi video?',...
'Create an AVI movie...','Yes','No','No');
if strcmp(button,'Yes')
   record_avi = 'y';
   [aviFile,aviPath] = uiputfile('*.avi','Save Video As');
   if isequal(aviFile,0)|isequal(aviPath,0)
       %% the dialog was canceld and we have no file name for the video
       %% thus no video will created
        msgbox('You canceld. No avi-file will be created!','Warning','warn');
        record_avi = 'n';
   else
       %% we have a corrct file name and will create the video file now
        mov = avifile([aviPath aviFile '.avi']);
        mov.Compression = 'None';
        mov.Quality = 100;
        movie_window = figure;
   end;
elseif strcmp(button,'No')
    %% the user wants no avi file
    record_avi = 'n';
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%% INPUT VARIABLES %%%%%%%%%%
%%%%%%%%%%%%%%%%
%%% GEOMETRY %%%
prompt = {'Electrode distance:',...
        'Number of electrodes:',...
        'Start points of electrodes (in m)',...
        'End points of electrodes (in m)',...
        'Length of channel (in m)',...
        'Width of channel/electrodes (in m)',...
        'Effective area correction (based on roughness)',...
        'Thickness of membrane (in m)'};
dlg_title = 'Geometry input';
num_lines = 1;
def     = {'6.5','1','[10]','[2010]','2200','800','1','10'};
answer  = inputdlg(prompt,dlg_title,num_lines,def);

% retrieve values from answer cell
d = str2num(answer{1})*1e-6;            % electrode distance [d] = m  (=z)
z_size = d;                             % diffusion layer width in m
nAE = str2num(answer{2});               % number of working electrodes
x_electrode = str2num(answer{3})*1e-6;  % start of electrode in m
X_electrode = str2num(answer{4})*1e-6;  % end of electrode in m
X = str2num(answer{5})*1e-6;            % length (of channel) of calculation area in m
y_size = str2num(answer{6})*1e-6;       % width of channel and electrode in m
A_eff = str2num(answer{7});
b = str2num(answer{8})*1e-6;            % thickness of membrane in m
% free answer
clear answer;

%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% CHEMICAL PROPERTIES %%%
% only for analyte
if analyte_yes_no ~= 0
    prompt = {'Diffusion coeff. of membrane (in cm/s):',...
        'Analyte concentration (in mol/l)' };
    dlg_title = 'Chemical properties of analyte...';
    num_lines = 1;
    def     = {'9E-6','1E-6'};
    answer  = inputdlg(prompt,dlg_title,num_lines,def);
    % retrieve values from answer
    D_m = str2num(answer{1})*1e-4;      % diffusion coeff of membrane for the analyte in cm/s
    c_analyt = str2num(answer{2})*1e+3; % concentration of the external analyte in mol/l
    P = D_m/b;
    clear answer;
else
    P = 0;
    c_analyt = 0;
    D_m = 0;
end;

% general chem. properties
prompt = {'Bulk concentration (in mol/l)',...
        'Concentration factor ( Ox/Red )',...
        'Diffusion coeff. of redoxmediator (in cm/s)',...
        'Charge transfered',...
        'Activity coeff. gamma for oxidized component',...
        'Activity coeff. gamma for reduced component'};
dlg_title = 'Chemical properties...';
num_lines = 1;
def     = {'0.1','1000000','6E-6','1','1','1'};
answer  = inputdlg(prompt,dlg_title,num_lines,def);

% retrieve values from answer cell
c_bulk = str2num(answer{1})*1e+3;   % concentration species A+B [c] = mol/m
ox_red = str2num(answer{2});        % relative concentration c_ox/c_red
c_0 = ox_red * c_bulk/(1+ox_red);   % concentration in bulk outside the calculation area
D = str2num(answer{3})*1e-4;        % diffusion constant    [D] = m / s
% % D_ox = 0.65e-5;0.71e-5
% % D_red = 0.76e-5;
charge = str2num(answer{4});
zFRT = FRT * charge;
gamma_ox = str2num(answer{5});
gamma_red = str2num(answer{6});
g = gamma_ox / gamma_red;
% free answer
clear answer;

% if ox_red is larger than 1, the simulated component is the majority component
% thus the analyte intake has to be subtracted from it, as we assume that the
% analyte will always react with the minority component
if ox_red > 1
    analyte_yes_no = -1 * analyte_yes_no;
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% VOLTAGES and POTENTIALS %%%
prompt = {'Standardpotential E0 (in mV)',...
          'Applied voltage (in mV)'};
dlg_title = 'Electrical properties...';
num_lines = 1;
def     = {'255','50'};
answer  = inputdlg(prompt,dlg_title,num_lines,def);

% retrieve values from answer cell
E0 = str2num(answer{1})*1e-3;            % standard potential in V
U  = str2num(answer{2})*1e-3;       % standard potential in V
% free answer
clear answer;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% DETERMINE START POTENTIALS %%%
if ox_red == 0 
    E_eq = E0-0.3;
else
    E_eq = E0 + 1/zFRT*log(g*ox_red);
end
E_step = 0.001;

%%%%%%%%%%%%%%%%%%%%%%%%%
%%% SOLVER PROPERTIES %%%
prompt = {'Simulation time (in s)',...
        'Automatically limit simulation time? (y/n)',...
        'Record concentration values at (in s)',...
        'Voltage step time (in s)',...
        'Minimum number of grid points in z-direction',...
        'Maximum time increment (in s)',...
        'Maximum length increment (in m)',...
        'Lambda (stability coeff.)',...
        'Output file name (t,I)'};
dlg_title = 'Solver properties...';
num_lines = 1;
def     = {'3','n','[0 0.0001 0.001 0.01 0.1 0.5 1 1.5 2 2.5 3]','0.0001','10','0.1','0.5','0.24','simxx.csv'};
answer  = inputdlg(prompt,dlg_title,num_lines,def);

% retrieve values from answer cell
t_sim = str2num(answer{1});         % simulation time in s
t_limit = answer{2};                % auto limit on calculation steps
t_record = str2num(answer{3});      % time points at which concentration profiles will be saved
vStepTime = str2num(answer{4});     % voltage step time in s
geom_step_min = str2num(answer{5}); % minimum number of geometry points
dt_max = str2num(answer{6});        % maximum time increment in s
dx_max = str2num(answer{7})*1e-6;   % maximum length increment in cm
lambda = str2num(answer{8});        % should be smaller than 0.25 to ensure stability
filename = answer{9};
% free answer
clear answer;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% determine grid and initialize variables %%%
%%%%%%%%%%%%%%%%%%%%%%%
%%% CALCULATE  GRID %%%
% the used algorithm should provide a good balance between number of points for
% small geometries and accuracy (maximum increment) for large geometries
% calculate dy using minimum number of geometrie points
dz = z_size/(geom_step_min);
% limit to maximum increment
if dz > dx_max
    dz = dx_max;
end;
% calculate dx using minimum number of geometrie points
dx = X/(geom_step_min);
% limit to maximum increment
if dx > dx_max
    dx = dx_max;
end;
% we use same increments in x and y direction, so we will use the smaller of both
if dx > dz
    dx = dz;
else
    dz = dx;
end;
% determine number of layers and round them towards infinity
n_layers_x = ceil(X      / dx);
n_layers_z = ceil(z_size / dz);

x_steps = n_layers_x + 1;         % number of points in x direction
z_steps = n_layers_z + 1;         % number of points in z direction

x = dx * (0:n_layers_x);
z = dz * (0:n_layers_z);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% CALCULATE ELECTRODE VECTORS %%%
% create an area array for easier computation of currents later on
% first we create an array that includes only radii covering the electrode
% areas
xAE = [];
XAE = [];
x_help = zeros(1,length(x));
for n = 1:nAE
    xAE(n) = length(x(find(x<=x_electrode(n))));
    XAE(n) = length(x(find(x<=X_electrode(n))));
    
    x_help(xAE(n):XAE(n)) = 1;
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% CALCULATE ELECTRODE AREA VECTORS %%%
% we will calculate current using areas centered on the geometry point.
% so we have to adjust the inner most and outer most area by dividing it by 2
% [A] = cm
A = zeros(1,x_steps);
A = y_size * dx * x_help;

% half first and last electrode area elements
for n = 1:nAE
    A(xAE(n)) = A(xAE(n))/2;
    A(XAE(n)) = A(XAE(n))/2;
end;

% correct area size for each element with A_eff
A = A * A_eff;
A_ges = sum(A);

%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% CALCULATE TIME GRID %%%
% determine time increment from betha criterium and limit to maximum time increment
dt = lambda * dx * dx / D;
if dt > dt_max
    dt = dt_max;
end
n_step = ceil(t_sim/dt);
% limit n_step
if t_limit == 'y'
    if n_step > z_steps*x_steps
        n_step = z_steps*x_steps;
    end;
end;

% time step at which the voltage on the electrodes will step
voltage_step = ceil(vStepTime/dt);

% recalculate lambda
lambda = D * dt / dx^2;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% INITIALIZE OUTPUT TIME VARIABLES %%%
% find t_record values in time vector
t_record = ceil(t_record./dt);
% make sure there is no zero index in the array
if t_record(1) == 1
    t_record(1) = 1;
end;

t_save = zeros(1,n_step);
t_save(t_record) = 1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% INITIALIZE CONCENTRATION VARIABLES %%%
% create start concentration vectors
c_old = ox_red * c_bulk/(1+ox_red) * ones(z_steps,x_steps);
c_new = c_old;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% INITIALIZE CURRENT VARIABLES %%%
% initialize current vector
I_redox_n = [];
current = 0;
% concentration profile history
C_slit = [];
% variable to keep track of peak current
curr_max = 0;
%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% INITIALIZE COUNTERS %%%
% we need two counter arrays for calculating in z and x direction
% for this we will use j and i respectively
j_  = 1:x_steps;             % includes all points
jj_ = 2:(x_steps-1);         % includes all but the outer most points (boundaries)
i_  = 1:z_steps;
ii_ = 2:(z_steps-1);
% boundary with electrode area (create array with all electrode indexes)
electrode_ = [];
for n = 1:nAE
    electrode_  = [electrode_ xAE(n):XAE(n)];
end;
% boundary without electrode area, exclusive the last value wich has
% to be calculated differently as it is a corner
inert_ = [];
if xAE(1) > 1
    inert_ = [2:(xAE(1)-1)];
    for n = 2:nAE
        inert_ = [inert_ (XAE(n-1)+1):(xAE(n)-1)];
    end;
    inert_ = [inert_ (XAE(nAE)+1):(x_steps-1)];
else
    for n = 2:nAE
        inert_ = [inert_ (XAE(n-1)+1):(xAE(n)-1)];
    end;
    inert_ = [inert_ (XAE(nAE)+1):(x_steps-1)];
end;

% display a wait bar to indicate calculation progress
h_waitbar = waitbar(0,'Fortschritt');

% % initialize electrode potentials
E1_n = [];
E2_n = [];
% initialize equilibrium potential
E_eq_n = [];
% initialize ox_red relative concentration
ox_red_n = [];

%%%% test
danalyt=[];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% START MAIN CALCULATION %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for l = 1:(n_step)
   %%%%%%%%%%%%%%%%%%%%%%%%%
   %%% CALCULATE CURRENT %%%
       % check on which electrode current will be smallest
   if l > voltage_step
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %%% calculate new electrode potentials %%%
        %%% these values are not needed for further calculation
        %%% and are kept only for recording/testing purposes
        c_ox_mean = mean(mean(c_old));
        ox_red = c_ox_mean / (c_bulk - c_ox_mean);
        E_eq=E0 + 1/zFRT*log(g*ox_red);
        
        K = sum( A.*(c_old(2,:) + c_old(z_steps-1,:)) ) / A_ges / c_bulk;
        % with g = gamma_ox/gamma_red
        E2 = (zFRT*E0-log(-1/2*(-1-exp(zFRT*U)+K+K*exp(zFRT*U)-(1+2*exp(zFRT*U)-2*K+4*K*exp(zFRT*U)+(exp(zFRT*U))^2-2*K*(exp(zFRT*U))^2+K^2-2*K^2*exp(zFRT*U)+K^2*(exp(zFRT*U))^2)^(1/2))/(K*exp(zFRT*U)*g)))/zFRT;
        E1 = E2 - U;
      Ei_1 = E1;
      Ei_2 = E2;
      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      %%%%%%%% dimensionless potential xi on electrode I %%%%%%%%
      xi = (zFRT * (Ei_1 - E0));
      % nernst charge transfer on boundary I
      c_old(1,electrode_) = c_bulk * exp(xi)/(g+exp(xi));
      c_new(1,electrode_) = c_old(1,electrode_);
    
      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      %%%% potential step and dimensionless potential on electrode II %%%%
      xi = (zFRT * (Ei_2 - E0));
      % nernst charge transfer on boundary II
      c_old(z_steps,electrode_) = c_bulk * exp(xi)/(g+exp(xi));
      c_new(z_steps,electrode_) = c_old(z_steps,electrode_);
    
      %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      %%%%%%%%%%%%%%  calculate resulting currents  %%%%%%%%%%%%%%%%%%%%%%
      % @ z = 0:
      % assuming tendency to uniform surface potential/concentration
      current = F*D/dz*sum(A .* (c_new(2,:) - c_new(1,:)));
      % @ z = max:
      % current on other electrode should be of same value
%       current2 = -F*D/dz*sum(A .* (c_new(z_steps,:) - c_new(z_steps-1,:)));

      %%% record electrode potentials %%%
      E1_n = [E1_n E1];
      E2_n = [E2_n E2];
      %%% record equilibrium potential %%%
      E_eq_n = [E_eq_n E_eq];
      %%% record ox_red relative concentration %%%
      ox_red_n = [ox_red_n ox_red];
      % calculate dimensionless current
      %current = current / (F*A_electrode*c_bulk*sqrt(F*E_step/t_step*D/R/T));
    end;
  
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%% CALCULATE DIFFUSION  %%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%  BOUNDARY I and II  %%%%%%
    % first calculate for the inert part of the boundaries in x direction
    % calculate CORNER POINTS (I-II) and (II-III)
    if xAE(1) > 1
        % there is no electrode!!!
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %%% CORNER (I-II): boundary at z = 0 (least index in matlab is 1!!!)
        c_new(1,1) = c_old(1,1) + lambda*( -4*c_old(1,1) + 2*c_old(1,2) + 2*c_old(2,1) ) + analyte_yes_no*1e3*P*dt/dx*c_analyt;
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %%% CORNER (II-III): boundary at z=max
        c_new(z_steps,1) = c_old(z_steps,1) + lambda*( 2*c_old(z_steps-1,1) - 4*c_old(z_steps,1) + 2*c_old(z_steps,2) ) + analyte_yes_no*1e3*P*dt/dx*c_analyt;
    else
        % there is electrode at the very beginning of the channel
        % add analyte input to already calculated c_new values (have been
        % calculated in the electrode section
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %%% CORNER (I-II): boundary at z = 0 (least index in matlab is 1!!!)
        c_new(1,1) = c_new(1,1) + analyte_yes_no*1e3*P*dt/dx*c_analyt;
         %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %%% CORNER (II-III): boundary at z=max
        c_new(z_steps,1) = c_new(z_steps,1) + analyte_yes_no*1e3*P*dt/dx*c_analyt;
    end;
    
    % inert_ holds all points without electrode exclusive corner points
    for j = inert_
        %%%%%%%%%%%%%%%%%%
        %%% BOUNDARY I %%%
        % boundary at z = 0 (least index in matlab is 1!!!)
        c_new(1,j) = c_old(1,j) + lambda*( c_old(1,j-1) - 4*c_old(1,j) + c_old(1,j+1) + 2*c_old(2,j) );
        %%%%%%%%%%%%%%%%%%%
        %%% BOUNDARY II %%%
        % boundary at z = max
        c_new(z_steps,j) = c_old(z_steps,j) + lambda*( c_old(z_steps,j-1) + 2*c_old(z_steps-1,j) - 4*c_old(z_steps,j) + c_old(z_steps,j+1) );        
    end;

    % calculate CORNER POINTS at x = max
    %%%%%%%%%%%%%%%%%%%%%%%%%
    %%% CORNER (I-IV): boundary at z = 0 (least index in matlab is 1!!!)
    c_new(1,x_steps) = c_old(1,x_steps) + lambda*( c_old(1,x_steps-1) - 4*c_old(1,x_steps) + c_0 + 2*c_old(2,x_steps) );
    %%%%%%%%%%%%%%%%%%%%%%%%%
    %%% CORNER (II-IV): boundary at z = max
    c_new(z_steps,x_steps) = c_old(z_steps,x_steps) + lambda*( c_old(z_steps,x_steps-1) + 2*c_old(z_steps-1,x_steps) - 4*c_old(z_steps,x_steps) + c_0 );        
    
    % loop through all layers in z direction with exeption of boundaries, which
    % have been treated already
    % calculate first element in x direction, if neccessary include boundary
    % condition here or inside loop
    %%%%%%%%%%%%%%%%%%
    %%% BOUNDARY III %%%
    %inert wall and analyte input
danalyt = [danalyt analyte_yes_no*1e3*P*dt/dx*c_analyt];

    c_new(ii_,1) = c_old(ii_,1) + lambda*( c_old(ii_-1,1) - 4*c_old(ii_,1) + 2*c_old(ii_,2) + c_old(ii_+1,1) ) + analyte_yes_no*1e3*P*dt/dx*c_analyt;
    for ii = ii_
        % calculate first element in x direction, if neccessary include boundary 1
        % condition here or before looping
        % loop through all points in x direction with exception of boundaries
        for jj = jj_
            %%%%%%%%%%%%%%%%%%
            %%% MAIN AREA: %%%
            c_new(ii,jj) = c_old(ii,jj) + lambda*( c_old(ii-1,jj) + c_old(ii,jj-1) - 4*c_old(ii,jj) + c_old(ii+1,jj) + c_old(ii,jj+1) );
        end;
        % calculate last element in x direction, if neccessary include
        % boundary IV condition here or before looping
    end;
    % calculate last element in x direction, if neccessary include boundary
    % condition here or inside loop
    %%%%%%%%%%%%%%%%%%
    %%% BOUNDARY IV %%%
    % steady concentration on bulk interface
    c_new(ii_,x_steps) = c_old(ii_,x_steps) + lambda*( c_old(ii_-1,x_steps) + c_old(ii_,x_steps-1) - 4*c_old(ii_,x_steps) + c_old(ii_+1,x_steps) + c_0 );

    c_old = c_new;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%% AVI %%%%%%%%%%%%%%%%%%%%%
if record_avi == 'y'
    figure(movie_window);
    mesh_c_red = mesh(10000*x,10000*z,c_old);
    set(mesh_c_red,'FaceColor','interp');
    set(mesh_c_red,'EdgeColor','interp');
    set(mesh_c_red,'FaceLighting','phong');
    set(mesh_c_red,'EdgeLighting','phong');
    title(['Konzentrationsverteilung C_{red} ber Zeit und Ort im Kapillarspalt Schritt ' num2str(l)]);
    ylabel('Ort im Spalt d in m');
    xlabel('Ort im Spalt x in m');
    zlabel('Konzentration c_{red} in mol/l');
    zlim([0 2.1e-7]);
    next_frame = getframe(gcf);
    mov = addframe(mov,next_frame);
end;
%%%%%          end avi             %%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%% RECORD VALUES %%%%%%
    %%% record current value %%%
    I_redox_n = [I_redox_n current];
    %%% record peak current %%%
    if current > curr_max
        curr_max = current;
    end;
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%% SAVE CONCENTRATION VALUES OVER TIME %%%
    if t_save(l) == 1
        % concatenate new concentration profile to history
        C_slit = cat(3,C_slit,c_new);
    end;
    % save last profile in C_end
    if l >= n_step
        C_end = c_new;
    end;
    
    % advance progress bar
    waitbar(l/(n_step+1),h_waitbar);
end;
%%%    END OF MAIN CALCULATION    %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% close recorded avi %%%
if record_avi == 'y'
    mov = close(mov)
end;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%  output results  %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% create named plot windows
scrsz = get(0,'ScreenSize');
scrsz(4) = (scrsz(4));
height=0.8*scrsz(4)/2;
fig_display2 = figure('name','concentration profile'            ,'Position',[1          60                  scrsz(3)/2 height]);
fig_display3 = figure('name','concentration profile at end time','Position',[scrsz(3)/2 60                  scrsz(3)/2 height]);
% fig_control  = figure('name','control'                        ,'Position',[1          scrsz(4)/2+1 scrsz(3)/2 height]);
fig_display1 = figure('name','current over time'                ,'Position',[scrsz(3)/2 scrsz(4)/2+1        scrsz(3)/2 height]);

% plot current over time
figure(fig_display1);
t = dt * (0:n_step-1);
plot(t,I_redox_n,'red');
text(0.7*t(n_step),curr_max,['I_{end}  = ' num2str(I_redox_n(n_step)) ' A' 10 'd      = ' num2str(1e6*d) ' m' 10 'A_{AE} = ' num2str(sum(1e4*A)) ' cm'],'HorizontalAlignment','left','VerticalAlignment','top');
xlabel('Zeit t/s');
ylabel('Strom I/A');
title('Strom in der Kapillarspaltzelle');

% plot concentration profile as contour plot
figure(fig_display2);
[C,h] = contourf(1e6*x,1e6*z,C_end,61);
set(h(:),'EdgeColor','none');
colorbar;

% plot the concentration profile over time using 3d graphics
figure(fig_display3);
% mesh_c_red = mesh(1e6*z,t,C_red);
% mesh_c_red = mesh(1e6*x,1e6*z,c_bulk-C_end);
mesh_c_red = mesh(1e6*x,1e6*z,C_end);
% set fac color to interpolated data points
set(mesh_c_red,'FaceColor','interp');
set(mesh_c_red,'EdgeColor','interp');
set(mesh_c_red,'FaceLighting','phong');
set(mesh_c_red,'EdgeLighting','phong');
title('Konzentrationsverteilung C_{red} ber Zeit und Ort im Kapillarspalt');
ylabel('Ort im Spalt d in m');
xlabel('Ort im Spalt x in m');
zlabel('Konzentration c_{ox} in mol/m');

% save data to file
fff=[t.' I_redox_n.'];
save(filename,'fff','-ascii');

% waitbar(1,h_waitbar);
close(h_waitbar);

